package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceInstitutionDurationEnt;
import pps.core.space.entity.SpaceInstitutionInstitutionEnt;
import pps.core.space.entity.SpaceInstitutionInstitutionView;
import pps.core.space.entity.SpaceInstitutionWellheadView;
import pps.core.space.mapper.SpaceInstitutionDurationMapper;
import pps.core.space.mapper.SpaceInstitutionInstitutionMapper;
import pps.core.space.mapper.SpaceInstitutionInstitutionViewMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadViewMapper;
import pps.core.space.service.data.space_institution_institution.*;
import pps.core.space.service.data.space_institution_wellhead.CreateSpaceInstitutionWellheadInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 基础间开配置模块
 *
 * @author ZWT
 * @date 2023/09/05
 */
@XService
@XText("基础间开配置模块")
public class SpaceInstitutionInstitutionService {

    /**
     * 基础间开配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--新增")
    public XServiceResult createSpaceInstitutionInstitution(XContext context, CreateSpaceInstitutionInstitutionInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            //新增间开配置
            SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
            SpaceInstitutionInstitutionEnt entity = XCopyUtils.copyNewObject(input, SpaceInstitutionInstitutionEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String institutionId = entity.getId();
            List<CreateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
            if (CollUtil.isNotEmpty(wellheadList)) {
                List<SpaceInstitutionDurationEnt> durationList;
                SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
                List<SpaceInstitutionWellheadView> wellheads = XCopyUtils.copyNewList(wellheadList, SpaceInstitutionWellheadView.class);
                for (SpaceInstitutionWellheadView wellhead : wellheads) {
                    wellhead.setInstitutionId(institutionId);
                    BaseUtils.setBaseModelDefault(wellhead, session);
                    wellheadMapper.insert(wellhead);
                    String wellheadId = wellhead.getWellheadId();
                    String configId = wellhead.getId();
                    durationList = wellhead.getDurationList();
                    if (CollUtil.isNotEmpty(durationList)) {
                        durationList.forEach(d -> {
                            d.setWellheadId(wellheadId);
                            d.setConfigId(configId);
                            d.setInstitutionId(institutionId);
                            BaseUtils.setBaseModelDefault(d, session);
                        });
                        BaseUtils.batchInsert(SpaceInstitutionDurationMapper.class, SpaceInstitutionDurationEnt.class, durationList);
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSpaceInstitutionInstitution(XContext context, UpdateSpaceInstitutionInstitutionInput input) {
        SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
        QueryWrapper<SpaceInstitutionInstitutionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionInstitutionEnt::getId, input.getId());
        SpaceInstitutionInstitutionEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSpaceInstitutionInstitution(XContext context, DeleteSpaceInstitutionInstitutionInput input) {
        SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
        QueryWrapper<SpaceInstitutionInstitutionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionInstitutionEnt::getId, input.getId());
        SpaceInstitutionInstitutionEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceInstitutionInstitutionOutput> getSpaceInstitutionInstitution(XContext context, GetSpaceInstitutionInstitutionInput input) {
        SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
        QueryWrapper<SpaceInstitutionInstitutionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionInstitutionEnt::getId, input.getId());
        SpaceInstitutionInstitutionEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionInstitutionOutput output = new GetSpaceInstitutionInstitutionOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionInstitutionOutput> querySpaceInstitutionInstitution(XContext context, QuerySpaceInstitutionInstitutionInput input) {
        SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
        QueryWrapper<SpaceInstitutionInstitutionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SpaceInstitutionInstitutionEnt::getId, input.getId())
                .eq(SpaceInstitutionInstitutionEnt::getIsDeleted, input.getIsDeleted())
                .like(SpaceInstitutionInstitutionEnt::getCreateById, input.getCreateById())
                .like(SpaceInstitutionInstitutionEnt::getCreateByName, input.getCreateByName())
                .eq(SpaceInstitutionInstitutionEnt::getCreateTime, input.getCreateTime())
                .like(SpaceInstitutionInstitutionEnt::getModifyById, input.getModifyById())
                .like(SpaceInstitutionInstitutionEnt::getModifyByName, input.getModifyByName())
                .eq(SpaceInstitutionInstitutionEnt::getModifyTime, input.getModifyTime())
                .like(SpaceInstitutionInstitutionEnt::getOuId, input.getOuId())
                .like(SpaceInstitutionInstitutionEnt::getLineId, input.getLineId())
                .like(SpaceInstitutionInstitutionEnt::getGridTypeKey, input.getGridTypeKey())
                .like(SpaceInstitutionInstitutionEnt::getInstitutionName, input.getInstitutionName())
                .eq(SpaceInstitutionInstitutionEnt::getInstitutionStartDate, input.getInstitutionStartDate())
                .eq(SpaceInstitutionInstitutionEnt::getInstitutionEndDate, input.getInstitutionEndDate())
                .eq(SpaceInstitutionInstitutionEnt::getOptimizeDeadline, input.getOptimizeDeadline())
                .eq(SpaceInstitutionInstitutionEnt::getOptimizeState, input.getOptimizeState())
                .eq(SpaceInstitutionInstitutionEnt::getIsCurrentBasic, input.getIsCurrentBasic())
                .eq(SpaceInstitutionInstitutionEnt::getStartInterval, input.getStartInterval());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySpaceInstitutionInstitutionOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionInstitutionOutput> dynamicQuerySpaceInstitutionInstitution(XContext context, CustomQueryInput input) {
        SpaceInstitutionInstitutionMapper mapper = context.getBean(SpaceInstitutionInstitutionMapper.class);
        return XMapperHelper.query(mapper, input, SpaceInstitutionInstitutionEnt.class, QuerySpaceInstitutionInstitutionOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceInstitutionInstitutionViewOutput> getSpaceInstitutionInstitutionView(XContext context, GetSpaceInstitutionInstitutionViewInput input) {
        SpaceInstitutionInstitutionViewMapper mapper = context.getBean(SpaceInstitutionInstitutionViewMapper.class);
        SpaceInstitutionInstitutionView record = new SpaceInstitutionInstitutionView();
        XCopyUtils.copyObject(input, record);
        SpaceInstitutionInstitutionView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionInstitutionViewOutput output = new GetSpaceInstitutionInstitutionViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionInstitutionViewOutput> querySpaceInstitutionInstitutionView(XContext context, QuerySpaceInstitutionInstitutionViewInput input) {
        SpaceInstitutionInstitutionViewMapper mapper = context.getBean(SpaceInstitutionInstitutionViewMapper.class);
        SpaceInstitutionInstitutionView record = new SpaceInstitutionInstitutionView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceInstitutionInstitutionView> list = mapper.selectList(record);
        PageInfo<SpaceInstitutionInstitutionView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceInstitutionInstitutionViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceInstitutionInstitutionViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
